@@ -1208,7 +1208,7 @@ typedef struct _ImageStatistics
     //    Bilevel        Grayscale       GrayscaleMatte
     //    Palette        PaletteMatte    TrueColor
     //    TrueColorMatte ColorSeparation ColorSeparationMatte
-    ImageType       type ( void );
+    ImageType       type ( void ) const;
 
     // Print detailed information about the image
     void            verbose ( const bool verboseFlag_ );
