@@ -721,7 +721,7 @@ static Image *ReadCINImage(const ImageInfo *image_info,
     count=ReadBlob(image,length,pixels);
     if ((size_t) count != length)
       break;
-    (void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+    (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
       quantum_type,pixels,exception);
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
@@ -1163,7 +1163,7 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
     p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+    (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
       quantum_type,pixels,&image->exception);
     count=WriteBlob(image,length,pixels);
     if (count != (ssize_t) length)
