@@ -91,6 +91,9 @@
 */
 static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+  ExceptionInfo
+    *sans_exception;
+
   Image
     *image;
 
@@ -126,9 +129,17 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
   read_info=CloneImageInfo(image_info);
   (void) InvokeDelegate(read_info,image,"dng:decode",(char *) NULL,exception);
   image=DestroyImage(image);
-  (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s.pnm",
+  (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s.png",
     read_info->unique);
-  image=ReadImage(read_info,exception);
+  sans_exception=AcquireExceptionInfo();
+  image=ReadImage(read_info,sans_exception);
+  sans_exception=DestroyExceptionInfo(sans_exception);
+  if (image == (Image *) NULL)
+    {
+      (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s.ppm",
+        read_info->unique);
+      image=ReadImage(read_info,exception);
+    }
   (void) RelinquishUniqueFileResource(read_info->filename);
   if (image != (Image *) NULL)
     {
@@ -230,6 +241,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("ARW");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sony Alpha Raw Image Format");
   entry->module=ConstantString("DNG");
@@ -237,6 +249,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("DNG");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Digital Negative");
   entry->module=ConstantString("DNG");
@@ -244,6 +257,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("CR2");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Canon Digital Camera Raw Image Format");
   entry->module=ConstantString("DNG");
@@ -251,6 +265,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("CRW");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Canon Digital Camera Raw Image Format");
   entry->module=ConstantString("DNG");
@@ -258,6 +273,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("DCR");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Kodak Digital Camera Raw Image File");
   entry->module=ConstantString("DNG");
@@ -265,6 +281,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("ERF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Epson RAW Format");
   entry->module=ConstantString("DNG");
@@ -272,6 +289,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("KDC");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Kodak Digital Camera Raw Image Format");
   entry->module=ConstantString("DNG");
@@ -279,6 +297,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("K25");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Kodak Digital Camera Raw Image Format");
   entry->module=ConstantString("DNG");
@@ -286,6 +305,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("MRW");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sony (Minolta) Raw Image File");
   entry->module=ConstantString("DNG");
@@ -293,6 +313,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("NEF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Nikon Digital SLR Camera Raw Image File");
   entry->module=ConstantString("DNG");
@@ -300,6 +321,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("ORF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Olympus Digital Camera Raw Image File");
   entry->module=ConstantString("DNG");
@@ -307,6 +329,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("PEF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Pentax Electronic File");
   entry->module=ConstantString("DNG");
@@ -314,6 +337,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("RAF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Fuji CCD-RAW Graphic File");
   entry->module=ConstantString("DNG");
@@ -321,6 +345,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("SRF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sony Raw Format");
   entry->module=ConstantString("DNG");
@@ -328,6 +353,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("SR2");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sony Raw Format 2");
   entry->module=ConstantString("DNG");
@@ -335,6 +361,7 @@ ModuleExport unsigned long RegisterDNGImage(void)
   entry=SetMagickInfo("X3F");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sigma Camera RAW Picture File");
   entry->module=ConstantString("DNG");
