@@ -448,13 +448,6 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
         image->columns,image->rows);
       break;
     }
-    case JPEGCompression:
-    {
-      (void) FormatMagickString(buffer,MaxTextExtent,
-        "currentfile %lu %lu "PS3_JPEGCompression" ByteStreamDecodeFilter\n",
-        image->columns,image->rows);
-      break;
-    }
     case LZWCompression:
     {
       (void) FormatMagickString(buffer,MaxTextExtent,
@@ -496,6 +489,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
   switch (compression)
   {
     case NoCompression:
+    default:
     {
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
       if (status == MagickFalse)
@@ -509,7 +503,6 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     }
     case FaxCompression:
     case Group4Compression:
-    default:
     {
       if ((compression == FaxCompression) ||
           (LocaleCompare(CCITTParam,"0") == 0))
@@ -518,12 +511,6 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
         status=Huffman2DEncodeImage(image_info,image,mask_image);
       break;
     }
-    case JPEGCompression:
-    {
-      status=InjectImageBlob(image_info,image,mask_image,"jpeg",
-        &image->exception);
-      break;
-    }
     case LZWCompression:
     {
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
@@ -758,7 +745,6 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  { NonMaskedImageDict }",
       "  ifelse",
       "  stencil { 0 setgray imagemask } { image } ifelse",
-      "  grestore",
       "  sp { showpage } if",
       "} bind def",
       (char *) NULL
