@@ -259,7 +259,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,quantum_type,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -336,7 +336,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               exception);
             if (q == (PixelPacket *) NULL)
               break;
-            length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+            length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
               quantum_info,quantum_types[i],pixels,exception);
             if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
               break;
@@ -405,7 +405,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               exception);
             if (q == (PixelPacket *) NULL)
               break;
-            length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+            length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
               quantum_info,quantum_types[i],pixels,exception);
             if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
               break;
@@ -506,7 +506,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               exception);
             if (q == (PixelPacket *) NULL)
               break;
-            length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+            length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
               quantum_info,quantum_types[i],pixels,exception);
             if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
               break;
@@ -848,7 +848,7 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
       case NoInterlace:
       default:
       {
-        ViewInfo
+        CacheView
           *image_view;
           
         PixelPacket
@@ -919,7 +919,7 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
             break;
           for (i=0; i < (long) channels; i++)
           {
-            length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+            length=ExportQuantumPixels(image,(const CacheView *) NULL,
               quantum_info,quantum_types[i],pixels,&image->exception);
             count=WriteBlob(image,length,pixels);
             if (count != (ssize_t) length)
@@ -949,7 +949,7 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
             p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+            length=ExportQuantumPixels(image,(const CacheView *) NULL,
               quantum_info,quantum_types[i],pixels,&image->exception);
             count=WriteBlob(image,length,pixels);
             if (count != (ssize_t) length)
@@ -994,7 +994,7 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
             p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+            length=ExportQuantumPixels(image,(const CacheView *) NULL,
               quantum_info,quantum_types[i],pixels,&image->exception);
             count=WriteBlob(image,length,pixels);
             if (count != (ssize_t) length)
