@@ -57,6 +57,7 @@
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/module.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
 #include "magick/option.h"
@@ -65,11 +66,12 @@
 #include "magick/quantum-private.h"
 #include "magick/profile.h"
 #include "magick/resize.h"
+#include "magick/semaphore.h"
 #include "magick/splay-tree.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
-#include "magick/module.h"
+#include "magick/thread_.h"
 #include "magick/utility.h"
 #include "magick/module.h"
 #if defined(MAGICKCORE_TIFF_DELEGATE)
@@ -102,7 +104,7 @@ static const ExifInfo
     { EXIFTAG_FNUMBER, TIFF_RATIONAL, "exif:FNumber" },
     { EXIFTAG_EXPOSUREPROGRAM, TIFF_SHORT, "exif:ExposureProgram" },
     { EXIFTAG_SPECTRALSENSITIVITY, TIFF_ASCII, "exif:SpectralSensitivity" },
-    /* { EXIFTAG_ISOSPEEDRATINGS, TIFF_LONG, "exif:ISOSpeedRatings" }, */
+    { EXIFTAG_ISOSPEEDRATINGS, TIFF_SHORT, "exif:ISOSpeedRatings" }, 
     { EXIFTAG_OECF, TIFF_UNDEFINED, "exif:OptoelectricConversionFactor" },
     { EXIFTAG_EXIFVERSION, TIFF_UNDEFINED, "exif:ExifVersion" },
     { EXIFTAG_DATETIMEORIGINAL, TIFF_ASCII, "exif:DateTimeOriginal" },
@@ -119,7 +121,7 @@ static const ExifInfo
     { EXIFTAG_LIGHTSOURCE, TIFF_SHORT, "exif:LightSource" },
     { EXIFTAG_FLASH, TIFF_SHORT, "exif:Flash" },
     { EXIFTAG_FOCALLENGTH, TIFF_RATIONAL, "exif:FocalLength" },
-    { EXIFTAG_SUBJECTAREA, TIFF_SHORT, "exif:SubjectArea" },
+    /* { EXIFTAG_SUBJECTAREA, TIFF_SHORT, "exif:SubjectArea" }, */
     { EXIFTAG_MAKERNOTE, TIFF_UNDEFINED, "exif:MakerNote" },
     { EXIFTAG_USERCOMMENT, TIFF_UNDEFINED, "exif:UserComment" },
     { EXIFTAG_SUBSECTIME, TIFF_ASCII, "exif:SubSecTime" },
@@ -162,8 +164,14 @@ static const ExifInfo
 /*
   Global declarations.
 */
-static ExceptionInfo
-  *tiff_exception;
+static MagickThreadKey
+  tiff_exception;
+
+static SemaphoreInfo
+  *tiff_semaphore = (SemaphoreInfo *) NULL;
+
+static volatile MagickBooleanType
+  instantiate_key = MagickFalse;
 #endif
 
 /*
@@ -314,14 +322,19 @@ static void TIFFErrors(const char *module,const char *format,va_list error)
   char
     message[MaxTextExtent];
 
+  ExceptionInfo
+    *exception;
+
 #if defined(MAGICKCORE_HAVE_VSNPRINTF)
   (void) vsnprintf(message,MaxTextExtent,format,error);
 #else
   (void) vsprintf(message,format,error);
 #endif
   (void) ConcatenateMagickString(message,".",MaxTextExtent);
-  (void) ThrowMagickException(tiff_exception,GetMagickModule(),CoderWarning,
-    message,"`%s'",module);
+  exception=(ExceptionInfo *) MagickGetThreadValue(tiff_exception);
+  if (exception != (ExceptionInfo *) NULL)
+    (void) ThrowMagickException(exception,GetMagickModule(),CoderWarning,
+      message,"`%s'",module);
 }
 
 static void TIFFGetProfiles(TIFF *tiff,Image *image)
@@ -436,7 +449,7 @@ static void TIFFGetEXIFProperties(TIFF *tiff,Image *image)
       }
       case TIFF_LONG:
       {
-        uint16
+        uint32
           longy;
 
         if (TIFFGetField(tiff,exif_info[i].tag,&longy) != 0)
@@ -460,6 +473,9 @@ static void TIFFGetEXIFProperties(TIFF *tiff,Image *image)
       (void) SetImageProperty(image,exif_info[i].property,value);
   }
   TIFFSetDirectory(tiff,directory);
+#else
+  (void) tiff;
+  (void) image;
 #endif
 }
 
@@ -516,14 +532,19 @@ static void TIFFWarnings(const char *module,const char *format,va_list warning)
   char
     message[MaxTextExtent];
 
+  ExceptionInfo
+    *exception;
+
 #if defined(MAGICKCORE_HAVE_VSNPRINTF)
   (void) vsnprintf(message,MaxTextExtent,format,warning);
 #else
   (void) vsprintf(message,format,warning);
 #endif
   (void) ConcatenateMagickString(message,".",MaxTextExtent);
-  (void) ThrowMagickException(tiff_exception,GetMagickModule(),CoderWarning,
-    message,"`%s'",module);
+  exception=(ExceptionInfo *) MagickGetThreadValue(tiff_exception);
+  if (exception != (ExceptionInfo *) NULL)
+    (void) ThrowMagickException(exception,GetMagickModule(),CoderWarning,
+      message,"`%s'",module);
 }
 
 static tsize_t TIFFWriteBlob(thandle_t image,tdata_t data,tsize_t size)
@@ -645,7 +666,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  tiff_exception=exception;
+  (void) MagickSetThreadValue(tiff_exception,exception);
   error_handler=TIFFSetErrorHandler(TIFFErrors);
   warning_handler=TIFFSetWarningHandler(TIFFWarnings);
   tiff=TIFFClientOpen(image->filename,"r",(thandle_t) image,TIFFReadBlob,
@@ -691,6 +712,44 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_MINSAMPLEVALUE,&min_sample_value);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_MAXSAMPLEVALUE,&max_sample_value);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_PHOTOMETRIC,&photometric);
+    switch (photometric)
+    {
+      case PHOTOMETRIC_MINISBLACK:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","min-is-black");
+        break;
+      }
+      case PHOTOMETRIC_MINISWHITE:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","min-is-white");
+        break;
+      }
+      case PHOTOMETRIC_PALETTE:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","palette");
+        break;
+      }
+      case PHOTOMETRIC_RGB:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","RGB");
+        break;
+      }
+      case PHOTOMETRIC_CIELAB:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","CIELAB");
+        break;
+      }
+      case PHOTOMETRIC_SEPARATED:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","separated");
+        break;
+      }
+      default:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","unknown");
+        break;
+      }
+    }
     if (image->debug != MagickFalse)
       {
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Geometry: %ux%u",
@@ -703,51 +762,8 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           "Min sample value: %u",min_sample_value);
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
           "Max sample value: %u",max_sample_value);
-        switch (photometric)
-        {
-          case PHOTOMETRIC_MINISBLACK:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: MINISBLACK");
-            break;
-          }
-          case PHOTOMETRIC_MINISWHITE:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: MINISWHITE");
-            break;
-          }
-          case PHOTOMETRIC_PALETTE:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: PALETTE");
-            break;
-          }
-          case PHOTOMETRIC_RGB:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: RGB");
-            break;
-          }
-          case PHOTOMETRIC_CIELAB:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: CIELAB");
-            break;
-          }
-          case PHOTOMETRIC_SEPARATED:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: SEPARATED");
-            break;
-          }
-          default:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric interpretation: %u",photometric);
-            break;
-          }
-        }
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Photometric "
+          "interpretation: %s",GetImageProperty(image,"tiff:photometric"));
       }
     image->columns=(unsigned long) width;
     image->rows=(unsigned long) height;
@@ -798,7 +814,12 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         image->chromaticity.blue_primary.y=chromaticity[5];
       }
     TIFFGetProperties(tiff,image);
-    TIFFGetEXIFProperties(tiff,image);
+    option=GetImageOption(image_info,"tiff:ignore-exif");
+    if (option == (const char *) NULL)
+      TIFFGetEXIFProperties(tiff,image);
+    else
+      if (IsMagickTrue(option) == MagickFalse)
+        TIFFGetEXIFProperties(tiff,image);
     TIFFGetProfiles(tiff,image);
     /*
       Allocate memory for the image and pixel buffer.
@@ -836,6 +857,12 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
              sampling_factor);
            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
              "Sampling Factors: %s",sampling_factor);
+           if ((samples_per_pixel > 1) && (photometric == PHOTOMETRIC_YCBCR))
+             {
+               (void) TIFFSetField(tiff,TIFFTAG_JPEGCOLORMODE,
+                 JPEGCOLORMODE_RGB);
+               photometric=PHOTOMETRIC_RGB;
+             }
          }
 #endif
         break;
@@ -1044,7 +1071,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+          length=ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
             quantum_type,pixels,exception);
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
@@ -1097,7 +1124,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+          length=ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
             quantum_type,pixels,exception);
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
@@ -1152,7 +1179,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
                 case 4: quantum_type=AlphaQuantum; break;
                 default: quantum_type=UndefinedQuantum; break;
               }
-            length=ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+            length=ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
               quantum_type,pixels,exception);
             if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
@@ -1476,6 +1503,14 @@ ModuleExport unsigned long RegisterTIFFImage(void)
   MagickInfo
     *entry;
 
+  AcquireSemaphoreInfo(&tiff_semaphore);
+  if (instantiate_key == MagickFalse)
+    {
+      if (MagickCreateThreadKey(&tiff_exception) == MagickFalse)
+        ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+      instantiate_key=MagickTrue;
+    }
+  RelinquishSemaphoreInfo(tiff_semaphore);
   *version='\0';
 #if defined(TIFF_VERSION)
   (void) FormatMagickString(version,MaxTextExtent,"%d",TIFF_VERSION);
@@ -1576,6 +1611,13 @@ ModuleExport void UnregisterTIFFImage(void)
   (void) UnregisterMagickInfo("TIF");
   (void) UnregisterMagickInfo("TIFF");
   (void) UnregisterMagickInfo("TIFF64");
+  AcquireSemaphoreInfo(&tiff_semaphore);
+  if (instantiate_key != MagickFalse)
+    if (MagickDeleteThreadKey(tiff_exception) == MagickFalse)
+      ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  instantiate_key=MagickFalse;
+  RelinquishSemaphoreInfo(tiff_semaphore);
+  DestroySemaphoreInfo(&tiff_semaphore);
 }
 
 #if defined(MAGICKCORE_TIFF_DELEGATE)
@@ -1968,6 +2010,9 @@ static void TIFFSetEXIFProperties(TIFF *tiff,Image *image)
     }
   }
   /* (void) TIFFSetField(tiff,TIFFTAG_EXIFIFD,offset); */
+#else
+  (void) tiff;
+  (void) image;
 #endif
 }
 
@@ -2026,12 +2071,14 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
     endian,
     photometric;
 
+  uint32
+    rows_per_strip;
+
   unsigned char
     *pixels;
 
   unsigned long
-    lsb_first,
-    rows_per_strip;
+    lsb_first;
 
   /*
     Open TIFF file.
@@ -2045,7 +2092,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  tiff_exception=(&image->exception);
+  (void) MagickSetThreadValue(tiff_exception,&image->exception);
   error_handler=TIFFSetErrorHandler((TIFFErrorHandler) TIFFErrors);
   warning_handler=TIFFSetWarningHandler((TIFFErrorHandler) TIFFWarnings);
   switch (image_info->endian)
@@ -2096,6 +2143,9 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
     if ((LocaleCompare(image_info->magick,"PTIF") == 0) &&
         (GetPreviousImageInList(image) != (Image *) NULL))
       (void) TIFFSetField(tiff,TIFFTAG_SUBFILETYPE,FILETYPE_REDUCEDIMAGE);
+    if ((image->columns != (uint32) image->columns) ||
+        (image->rows != (uint32) image->rows))
+      ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
     (void) TIFFSetField(tiff,TIFFTAG_IMAGELENGTH,(uint32) image->rows);
     (void) TIFFSetField(tiff,TIFFTAG_IMAGEWIDTH,(uint32) image->columns);
     compression=image->compression;
@@ -2241,7 +2291,8 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
               photometric=PHOTOMETRIC_RGB;
             }
         (void) TIFFSetField(tiff,TIFFTAG_SAMPLESPERPIXEL,3);
-        if (image_info->type != TrueColorType)
+        if ((image_info->type != TrueColorType) &&
+            (image_info->type != TrueColorMatteType))
           {
             unsigned long
               depth;
@@ -2558,7 +2609,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,quantum_type,pixels,&image->exception);
               if (TIFFWritePixels(tiff,&tiff_info,y,0,image) == -1)
                 break;
@@ -2586,7 +2637,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,RedQuantum,pixels,&image->exception);
               if (TIFFWritePixels(tiff,&tiff_info,y,0,image) == -1)
                 break;
@@ -2606,7 +2657,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,GreenQuantum,pixels,&image->exception);
               if (TIFFWritePixels(tiff,&tiff_info,y,1,image) == -1)
                 break;
@@ -2626,7 +2677,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,BlueQuantum,pixels,&image->exception);
               if (TIFFWritePixels(tiff,&tiff_info,y,2,image) == -1)
                 break;
@@ -2647,7 +2698,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+                length=ExportQuantumPixels(image,(const CacheView *) NULL,
                   quantum_info,AlphaQuantum,pixels,&image->exception);
                 if (TIFFWritePixels(tiff,&tiff_info,y,3,image) == -1)
                   break;
@@ -2681,7 +2732,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
             quantum_info,quantum_type,pixels,&image->exception);
           if (TIFFWritePixels(tiff,&tiff_info,y,0,image) == -1)
             break;
@@ -2751,7 +2802,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
             quantum_info,quantum_type,pixels,&image->exception);
           if (TIFFWritePixels(tiff,&tiff_info,y,0,image) == -1)
             break;
