@@ -612,10 +612,13 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       MaxTextExtent);
     if (IsBlobTemporary(image) != MagickFalse)
       (void) CopyMagickString(next->filename,filename,MaxTextExtent);
-    value=GetImageProperty(next,"exif:Orientation");
+    value=GetImageProperty(next,"tiff:Orientation");
+    if (value == (char *) NULL)
+      value=GetImageProperty(next,"exif:Orientation");
     if (value != (char *) NULL)
       {
         next->orientation=(OrientationType) atol(value);
+        (void) DeleteImageProperty(next,"tiff:Orientation");
         (void) DeleteImageProperty(next,"exif:Orientation");
       }
     value=GetImageProperty(next,"tiff:XResolution");
@@ -741,10 +744,10 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       }
     (void) FormatMagickTime(GetBlobProperties(next)->st_mtime,MaxTextExtent,
       timestamp);
-    (void) SetImageProperty(next,"modify-date",timestamp);
+    (void) SetImageProperty(next,"date:modify",timestamp);
     (void) FormatMagickTime(GetBlobProperties(next)->st_ctime,MaxTextExtent,
       timestamp);
-    (void) SetImageProperty(next,"create-date",timestamp);
+    (void) SetImageProperty(next,"date:create",timestamp);
     if (read_info->verbose != MagickFalse)
       (void) IdentifyImage(next,stdout,MagickFalse);
     image=next;
