@@ -54,6 +54,7 @@
 #include "magick/monitor-private.h"
 #include "magick/pixel-private.h"
 #include "magick/quantum.h"
+#include "magick/transform.h"
 
 /*
   Define declarations.
@@ -189,7 +190,7 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     height,
     width;
 
-  ViewInfo
+  CacheView
     *frame_view;
 
   /*
@@ -397,7 +398,7 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     Draw sides of ornamental border.
   */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,8) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -658,7 +659,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     foreground,
     background;
 
-  ViewInfo
+  CacheView
     *image_view;
 
   assert(image != (Image *) NULL);
