@@ -823,7 +823,7 @@ static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
           if (magick_list == (SplayTreeInfo *) NULL)
             ThrowFatalException(ResourceLimitFatalError,
               "MemoryAllocationFailed");
-          magick_info=SetMagickInfo("tmp");
+          magick_info=SetMagickInfo("ephemeral");
           magick_info->stealth=MagickTrue;
           status=AddValueToSplayTree(magick_list,magick_info->name,magick_info);
           if (status == MagickFalse)
@@ -1055,7 +1055,7 @@ MagickExport MagickBooleanType IsMagickInstantiated(void)
 %
 %  MagickCoreGenesis() initializes the MagickCore environment.
 %
-%  The format of the InitializeMagick function is:
+%  The format of the MagickCoreGenesis function is:
 %
 %      MagickCoreGenesis(const char *path,
 %        const MagickBooleanType establish_signal_handlers)
@@ -1166,6 +1166,8 @@ MagickExport void MagickCoreGenesis(const char *path,
   /*
     Initialize the Magick environment.
   */
+  (void) setlocale(LC_ALL,"");
+  (void) setlocale(LC_NUMERIC,"C");
   InitializeSemaphore();
   seconds=time((time_t *) NULL);
   events=GetEnvironmentValue("MAGICK_DEBUG");
