@@ -89,7 +89,7 @@
 %
 %  The format of the ExportQuantumPixels method is:
 %
-%      size_t ExportQuantumPixels(const Image *image,const ViewInfo *cache_view,
+%      size_t ExportQuantumPixels(const Image *image,const CacheView *image_view,
 %        const QuantumInfo *quantum_info,const QuantumType quantum_type,
 %        unsigned char *pixels,ExceptionInfo *exception)
 %
@@ -97,7 +97,7 @@
 %
 %    o image: the image.
 %
-%    o cache_view: the image cache view.
+%    o image_view: the image cache view.
 %
 %    o quantum_info: the quantum info.
 %
@@ -233,7 +233,7 @@ static inline unsigned char *PopQuantumLongPixel(QuantumState *quantum_state,
 }
 
 MagickExport size_t ExportQuantumPixels(const Image *image,
-  const ViewInfo *cache_view,const QuantumInfo *quantum_info,
+  const CacheView *image_view,const QuantumInfo *quantum_info,
   const QuantumType quantum_type,unsigned char *pixels,ExceptionInfo *exception)
 {
   EndianType
@@ -280,11 +280,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
   number_pixels=GetImageExtent(image);
   p=GetVirtualPixelQueue(image);
   indexes=GetVirtualIndexQueue(image);
-  if (cache_view != (ViewInfo *) NULL)
+  if (image_view != (CacheView *) NULL)
     {
-      number_pixels=GetCacheViewExtent(cache_view);
-      p=GetCacheViewVirtualPixelQueue(cache_view);
-      indexes=GetCacheViewVirtualIndexQueue(cache_view);
+      number_pixels=GetCacheViewExtent(image_view);
+      p=GetCacheViewVirtualPixelQueue(image_view);
+      indexes=GetCacheViewVirtualIndexQueue(image_view);
     }
   if (quantum_info->alpha_type == AssociatedQuantumAlpha)
     {
@@ -295,8 +295,8 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         Associate alpha.
       */
       q=GetAuthenticPixelQueue(image);
-      if (cache_view != (ViewInfo *) NULL)
-        q=GetAuthenticPixelQueue(image);
+      if (image_view != (CacheView *) NULL)
+        q=(PixelPacket *) GetCacheViewVirtualPixelQueue(image_view);
       for (x=0; x < (long) image->columns; x++)
       {
         alpha=QuantumScale*((double) QuantumRange-q->opacity);
@@ -306,6 +306,39 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         q++;
       }
     }
+  if ((quantum_type == RGBOQuantum) || (quantum_type == CMYKOQuantum))
+    {
+      register PixelPacket
+        *__restrict q;
+
+      q=GetAuthenticPixelQueue(image);
+      if (image_view != (CacheView *) NULL)
+        q=(PixelPacket *) GetCacheViewVirtualPixelQueue(image_view);
+      for (x=0; x < (long) number_pixels; x++)
+      {
+        q->opacity=(Quantum) (QuantumRange-q->opacity);
+        q++;
+      }
+    }
+  if ((quantum_type == CbYCrQuantum) || (quantum_type == CbYCrAQuantum))
+    {
+      Quantum
+        quantum;
+
+      register PixelPacket
+        *__restrict q;
+
+      q=GetAuthenticPixelQueue(image);
+      if (image_view != (CacheView *) NULL)
+        q=GetAuthenticPixelQueue(image);
+      for (x=0; x < (long) number_pixels; x++)
+      {
+        quantum=q->red;
+        q->red=q->green;
+        q->green=quantum;
+        q++;
+      }
+    }
   x=0;
   q=pixels;
   InitializeQuantumState(quantum_info,image->endian,&quantum_state);
@@ -840,28 +873,38 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
       {
         case 1:
         {
-          register unsigned char
-            pixel;
+          register Quantum
+            threshold;
 
+          register unsigned char
+            black,
+            pixel,
+            white;
+
+          black=0x00;
+          white=0x01;
+          if (quantum_info->min_is_white == MagickFalse)
+            {
+              black=0x01;
+              white=0x00;
+            }
+          threshold=(Quantum) (QuantumRange/2);
           for (x=((long) number_pixels-3); x > 0; x-=4)
           {
-            pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-            *q=(unsigned char) (((int) pixel != 0 ? 0x00 : 0x01) << 7);
+            *q='\0';
+            *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) << 7;
             pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 : 0x01);
             *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 6);
             p++;
-            pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-            *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 5);
+            *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) << 5;
             pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 : 0x01);
             *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 4);
             p++;
-            pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-            *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 3);
+            *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) << 3;
             pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 : 0x01);
             *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 2);
             p++;
-            pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-            *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 1);
+            *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) << 1;
             pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 : 0x01);
             *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 0);
             p++;
@@ -872,12 +915,12 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
               *q='\0';
               for (bit=3; bit >= (long) (4-(number_pixels % 4)); bit-=2)
               {
-                pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-                *q|=(((int) pixel != 0 ? 0x00 : 0x01) << (unsigned char) bit);
-                pixel=(unsigned char) (p->opacity == OpaqueOpacity ?
-                  0x00 : 0x01);
-                *q|=(((int) pixel != 0 ? 0x00 : 0x01) <<
-                  (unsigned char) (bit-1));
+                *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) <<
+                  bit;
+                pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 :
+                  0x01);
+                *q|=(((int) pixel != 0 ? 0x00 : 0x01) << (unsigned char)
+                  (bit-1));
                 p++;
               }
               q++;
@@ -1523,6 +1566,7 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
       break;
     }
     case RGBQuantum:
+    case CbYCrQuantum:
     {
       switch (quantum_info->depth)
       {
@@ -1779,6 +1823,8 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
       break;
     }
     case RGBAQuantum:
+    case RGBOQuantum:
+    case CbYCrAQuantum:
     {
       switch (quantum_info->depth)
       {
@@ -1900,120 +1946,6 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
       }
       break;
     }
-    case RGBOQuantum:
-    {
-      switch (quantum_info->depth)
-      {
-        case 8:
-        {
-          register unsigned char
-            pixel;
-
-          for (x=0; x < (long) number_pixels; x++)
-          {
-            pixel=ScaleQuantumToChar(p->red);
-            q=PopCharPixel(pixel,q);
-            pixel=ScaleQuantumToChar(p->green);
-            q=PopCharPixel(pixel,q);
-            pixel=ScaleQuantumToChar(p->blue);
-            q=PopCharPixel(pixel,q);
-            pixel=ScaleQuantumToChar(p->opacity);
-            q=PopCharPixel(pixel,q);
-            p++;
-            q+=quantum_info->pad;
-          }
-          break;
-        }
-        case 16:
-        {
-          register unsigned short
-            pixel;
-
-          for (x=0; x < (long) number_pixels; x++)
-          {
-            pixel=ScaleQuantumToShort(p->red);
-            q=PopShortPixel(endian,pixel,q);
-            pixel=ScaleQuantumToShort(p->green);
-            q=PopShortPixel(endian,pixel,q);
-            pixel=ScaleQuantumToShort(p->blue);
-            q=PopShortPixel(endian,pixel,q);
-            pixel=ScaleQuantumToShort(p->opacity);
-            q=PopShortPixel(endian,pixel,q);
-            p++;
-            q+=quantum_info->pad;
-          }
-          break;
-        }
-        case 32:
-        {
-          register unsigned long
-            pixel;
-
-          if (quantum_info->format == FloatingPointQuantumFormat)
-            {
-              for (x=0; x < (long) number_pixels; x++)
-              {
-                q=PopFloatPixel(&quantum_state,(float) p->red,q);
-                q=PopFloatPixel(&quantum_state,(float) p->green,q);
-                q=PopFloatPixel(&quantum_state,(float) p->blue,q);
-                q=PopFloatPixel(&quantum_state,(float) p->opacity,q);
-                p++;
-                q+=quantum_info->pad;
-              }
-              break;
-            }
-          for (x=0; x < (long) number_pixels; x++)
-          {
-            pixel=ScaleQuantumToLong(p->red);
-            q=PopLongPixel(endian,pixel,q);
-            pixel=ScaleQuantumToLong(p->green);
-            q=PopLongPixel(endian,pixel,q);
-            pixel=ScaleQuantumToLong(p->blue);
-            q=PopLongPixel(endian,pixel,q);
-            pixel=ScaleQuantumToLong(p->opacity);
-            q=PopLongPixel(endian,pixel,q);
-            p++;
-            q+=quantum_info->pad;
-          }
-          break;
-        }
-        case 64:
-        {
-          if (quantum_info->format == FloatingPointQuantumFormat)
-            {
-              for (x=0; x < (long) number_pixels; x++)
-              {
-                q=PopDoublePixel(&quantum_state,(double) p->red,q);
-                q=PopDoublePixel(&quantum_state,(double) p->green,q);
-                q=PopDoublePixel(&quantum_state,(double) p->blue,q);
-                q=PopDoublePixel(&quantum_state,(double) p->opacity,q);
-                p++;
-                q+=quantum_info->pad;
-              }
-              break;
-            }
-        }
-        default:
-        {
-          range=GetQuantumRange(image->depth);
-          for (x=0; x < (long) number_pixels; x++)
-          {
-            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->red,range),q);
-            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->green,range),q);
-            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->blue,range),q);
-            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->opacity,range),q);
-            p++;
-            q+=quantum_info->pad;
-          }
-          break;
-        }
-      }
-      break;
-    }
     case CMYKQuantum:
     {
       if (image->colorspace != CMYKColorspace)
@@ -2135,6 +2067,7 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
       break;
     }
     case CMYKAQuantum:
+    case CMYKOQuantum:
     {
       if (image->colorspace != CMYKColorspace)
         {
@@ -2272,8 +2205,156 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
       }
       break;
     }
+    case CbYCrYQuantum:
+    {
+     long
+        n;
+
+      Quantum
+        cbcr[4];
+
+      register long
+        i;
+
+      register unsigned long
+        pixel;
+
+      unsigned long
+        quantum;
+
+      n=0;
+      quantum=0;
+      range=GetQuantumRange(image->depth);
+      switch (quantum_info->depth)
+      {
+        case 10:
+        {
+          if (quantum_info->pack == MagickFalse)
+            {
+              for (x=0; x < (long) number_pixels; x+=2)
+              {
+                for (i=0; i < 4; i++)
+                {
+                  switch (n % 3)
+                  {
+                    case 0:
+                    {
+                      quantum=p->red;
+                      break;
+                    }
+                    case 1:
+                    {
+                      quantum=p->green;
+                      break;
+                    }
+                    case 2:
+                    {
+                      quantum=p->blue;
+                      break;
+                    }
+                  }
+                  cbcr[i]=(Quantum) quantum;
+                  n++;
+                }
+                pixel=(unsigned long) ((unsigned long) (cbcr[1]) << 22 |
+                  (unsigned long) (cbcr[0]) <<  12 |
+                  (unsigned long) (cbcr[2]) << 2);
+                q=PopLongPixel(endian,pixel,q);
+                p++;
+                pixel=(unsigned long) ((unsigned long) (cbcr[3]) << 22 |
+                  (unsigned long) (cbcr[0]) <<  12 |
+                  (unsigned long) (cbcr[2]) << 2);
+                q=PopLongPixel(endian,pixel,q);
+                p++;
+                q+=quantum_info->pad;
+              }
+              break;
+            }
+          break;
+        }
+        default:
+        {
+          for (x=0; x < (long) number_pixels; x+=2)
+          {
+            for (i=0; i < 4; i++)
+            {
+              switch (n % 3)
+              {
+                case 0:
+                {
+                  quantum=p->red;
+                  break;
+                }
+                case 1:
+                {
+                  quantum=p->green;
+                  break;
+                }
+                case 2:
+                {
+                  quantum=p->blue;
+                  break;
+                }
+              }
+              cbcr[i]=(Quantum) quantum;
+              n++;
+            }
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[1],range),q);
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[0],range),q);
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[2],range),q);
+            p++;
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[3],range),q);
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[0],range),q);
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[2],range),q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+      }
+      break;
+    }
     default:
       break;
   }
+  if ((quantum_type == CbYCrQuantum) || (quantum_type == CbYCrAQuantum))
+    {
+      Quantum
+        quantum;
+
+      register PixelPacket
+        *__restrict q;
+
+      q=GetAuthenticPixelQueue(image);
+      if (image_view != (CacheView *) NULL)
+        q=(PixelPacket *) GetCacheViewVirtualPixelQueue(image_view);
+      for (x=0; x < (long) number_pixels; x++)
+      {
+        quantum=q->red;
+        q->red=q->green;
+        q->green=quantum;
+        q++;
+      }
+    }
+  if ((quantum_type == RGBOQuantum) || (quantum_type == CMYKOQuantum))
+    {
+      register PixelPacket
+        *__restrict q;
+
+      q=GetAuthenticPixelQueue(image);
+      if (image_view != (CacheView *) NULL)
+        q=(PixelPacket *) GetCacheViewVirtualPixelQueue(image_view);
+      for (x=0; x < (long) number_pixels; x++)
+      {
+        q->opacity=(Quantum) (QuantumRange-q->opacity);
+        q++;
+      }
+    }
   return(extent);
 }
