@@ -193,7 +193,7 @@ MagickExport void DestroyImageRegistry(void)
 %
 %  The format of the GetImageRegistry method is:
 %
-%      const void *GetImageRegistry(const RegistryType type,const char *key,
+%      void *GetImageRegistry(const RegistryType type,const char *key,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -205,10 +205,10 @@ MagickExport void DestroyImageRegistry(void)
 %    o exception: the exception.
 %
 */
-MagickExport const void *GetImageRegistry(const RegistryType type,
-  const char *key,ExceptionInfo *exception)
+MagickExport void *GetImageRegistry(const RegistryType type,const char *key,
+  ExceptionInfo *exception)
 {
-  const void
+  void
     *value;
 
   RegistryInfo
@@ -217,7 +217,7 @@ MagickExport const void *GetImageRegistry(const RegistryType type,
   if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",key);
   if (registry == (void *) NULL)
-    return((const char *) NULL);
+    return((void *) NULL);
   registry_info=(RegistryInfo *) GetValueFromSplayTree(registry,key);
   if (registry_info == (void *) NULL)
     {
@@ -438,19 +438,23 @@ MagickExport MagickBooleanType SetImageRegistry(const RegistryType type,
     case StringRegistryType:
     default:
     {
-      clone_value=(void *) ConstantString((char *) value);
+      const char
+        *string;
+
+      string=(const char *) value;
+      clone_value=(void *) ConstantString(string);
       break;
     }
     case ImageRegistryType:
     {
-      Image
+      const Image
         *image;
 
-      image=(Image *) value;
+      image=(const Image *) value;
       if (image->signature != MagickSignature)
         {
           (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
-            "UnableToSetRegistry",key);
+            "UnableToSetRegistry","%s",key);
           return(MagickFalse);
         }
       clone_value=(void *) CloneImageList(image,exception);
@@ -458,14 +462,14 @@ MagickExport MagickBooleanType SetImageRegistry(const RegistryType type,
     }
     case ImageInfoRegistryType:
     {
-      ImageInfo
+      const ImageInfo
         *image_info;
 
-      image_info=(ImageInfo *) value;
+      image_info=(const ImageInfo *) value;
       if (image_info->signature != MagickSignature)
         {
           (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
-            "UnableToSetRegistry",key);
+            "UnableToSetRegistry","%s",key);
           return(MagickFalse);
         }
       clone_value=(void *) CloneImageInfo(image_info);
