@@ -581,6 +581,7 @@ static unsigned long ValidateImageFormatsInMemory(ImageInfo *image_info,
 #endif
       if (reference_image->colorspace != RGBColorspace)
         fuzz+=0.3;
+      fuzz+=MagickEpsilon;
       difference_image=CompareImageChannels(reference_image,reconstruct_image,
         AllChannels,MeanSquaredErrorMetric,&distortion,exception);
       reconstruct_image=DestroyImage(reconstruct_image);
@@ -783,6 +784,7 @@ static unsigned long ValidateImageFormatsOnDisk(ImageInfo *image_info,
 #endif
       if (reference_image->colorspace != RGBColorspace)
         fuzz+=0.3;
+      fuzz+=MagickEpsilon;
       difference_image=CompareImageChannels(reference_image,reconstruct_image,
         AllChannels,MeanSquaredErrorMetric,&distortion,exception);
       reconstruct_image=DestroyImage(reconstruct_image);
@@ -1179,6 +1181,7 @@ static MagickBooleanType ValidateUsage(void)
       "-debug events        display copious debugging information",
       "-help                print program options",
       "-log format          format of debugging information",
+      "-validate type       validation type",
       "-version             print version information",
       (char *) NULL
     },
@@ -1203,6 +1206,21 @@ static MagickBooleanType ValidateUsage(void)
 
 int main(int argc,char **argv)
 {
+#define DestroyValidate() \
+{ \
+  timer=DestroyTimerInfo(timer); \
+  image_info=DestroyImageInfo(image_info); \
+  exception=DestroyExceptionInfo(exception); \
+}
+#define ThrowValidateException(asperity,tag,option) \
+{ \
+  (void) ThrowMagickException(exception,GetMagickModule(),asperity,tag,"`%s'", \
+    option); \
+  CatchException(exception); \
+  DestroyValidate(); \
+  return(MagickFalse); \
+}
+
   char
     output_filename[MaxTextExtent],
     reference_filename[MaxTextExtent],
@@ -1236,50 +1254,112 @@ int main(int argc,char **argv)
     iterations,
     tests;
 
+  ValidateType
+    type;
+
   /*
     Validate the ImageMagick image processing suite.
   */
   MagickCoreGenesis(*argv,MagickFalse);
   iterations=1;
   status=MagickFalse;
+  type=AllValidate;
   regard_warnings=MagickFalse;
-  for (i=1; i < (long) argc; i++)
-  {
-    option=argv[i];
-    if ((strlen(option) == 1) || ((*option != '-') && (*option != '+')))
-      continue;
-    if (LocaleCompare("bench",option+1) == 0)
-      iterations=(unsigned long) atol(argv[++i]);
-    if (LocaleCompare("debug",option+1) == 0)
-      (void) SetLogEventMask(argv[++i]);
-    if (LocaleCompare("help",option+1) == 0)
-      {
-        (void) ValidateUsage();
-        return(0);
-      }
-    if (LocaleCompare("log",option+1) == 0)
-      {
-        if (*option != '+')
-          (void) SetLogFormat(argv[i+1]);
-      }
-    if (LocaleCompare("regard-warnings",option+1) == 0)
-      regard_warnings=MagickTrue;
-    if ((LocaleCompare("version",option+1) == 0) ||
-        (LocaleCompare("-version",option+1) == 0))
-      {
-        (void) fprintf(stdout,"Version: %s\n",
-          GetMagickVersion((unsigned long *) NULL));
-        (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
-        return(0);
-      }
-  }
-  timer=AcquireTimerInfo();
-  GetTimerInfo(timer);
   exception=AcquireExceptionInfo();
   image_info=AcquireImageInfo();
   (void) CopyMagickString(image_info->filename,ReferenceFilename,MaxTextExtent);
-  if (argc > 1)
-    (void) CopyMagickString(image_info->filename,argv[argc-1],MaxTextExtent);
+  timer=AcquireTimerInfo();
+  GetTimerInfo(timer);
+  for (i=1; i < (long) argc; i++)
+  {
+    option=argv[i];
+    if (IsMagickOption(option) == MagickFalse)
+      {
+        (void) CopyMagickString(image_info->filename,option,MaxTextExtent);
+        continue;
+      }
+    switch (*(option+1))
+    {
+      case 'b':
+      {
+        if (LocaleCompare("bench",option+1) == 0)
+          {
+            iterations=(unsigned long) atol(argv[++i]);
+            break;
+          }
+        ThrowValidateException(OptionError,"UnrecognizedOption",option)
+      }
+      case 'd':
+      {
+        if (LocaleCompare("debug",option+1) == 0)
+          {
+            (void) SetLogEventMask(argv[++i]);
+            break;
+          }
+        ThrowValidateException(OptionError,"UnrecognizedOption",option)
+      }
+      case 'h':
+      {
+        if (LocaleCompare("help",option+1) == 0)
+          {
+            (void) ValidateUsage();
+            return(0);
+          }
+        ThrowValidateException(OptionError,"UnrecognizedOption",option)
+      }
+      case 'l':
+      {
+        if (LocaleCompare("log",option+1) == 0)
+          {
+            if (*option != '+')
+              (void) SetLogFormat(argv[i+1]);
+            break;
+          }
+        ThrowValidateException(OptionError,"UnrecognizedOption",option)
+      }
+      case 'r':
+      {
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          {
+            regard_warnings=MagickTrue;
+            break;
+          }
+        ThrowValidateException(OptionError,"UnrecognizedOption",option)
+      }
+      case 'v':
+      {
+        if (LocaleCompare("validate",option+1) == 0)
+          {
+            long
+              validate;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowValidateException(OptionError,"MissingArgument",option);
+            validate=ParseMagickOption(MagickValidateOptions,MagickFalse,
+              argv[i]);
+            if (validate < 0)
+              ThrowValidateException(OptionError,"UnrecognizedValidateType",
+                argv[i]);
+            type=(ValidateType) validate;
+            break;
+          }
+        if ((LocaleCompare("version",option+1) == 0) ||
+            (LocaleCompare("-version",option+1) == 0))
+          {
+            (void) fprintf(stdout,"Version: %s\n",
+              GetMagickVersion((unsigned long *) NULL));
+            (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+            return(0);
+          }
+        ThrowValidateException(OptionError,"UnrecognizedOption",option)
+      }
+      default:
+        ThrowValidateException(OptionError,"UnrecognizedOption",option)
+    }
+  }
   reference_image=ReadImage(image_info,exception);
   tests=0;
   fail=0;
@@ -1305,32 +1385,41 @@ int main(int argc,char **argv)
             GetMagickVersion((unsigned long *) NULL));
           (void) fprintf(stdout,"Copyright: %s\n\n",
             GetMagickCopyright());
-          (void) fprintf(stdout,"ImageMagick Validation Suite\n\n");
-          tests+=ValidateCompareCommand(image_info,reference_filename,
-            output_filename,&fail,exception);
-          tests+=ValidateConvertCommand(image_info,reference_filename,
-            output_filename,&fail,exception);
-          tests+=ValidateCompositeCommand(image_info,reference_filename,
-            output_filename,&fail,exception);
-          tests+=ValidateIdentifyCommand(image_info,reference_filename,
-            output_filename,&fail,exception);
-          tests+=ValidateMontageCommand(image_info,reference_filename,
-            output_filename,&fail,exception);
-          tests+=ValidateStreamCommand(image_info,reference_filename,
-            output_filename,&fail,exception);
-          tests+=ValidateImageFormatsOnDisk(image_info,reference_filename,
-            output_filename,&fail,exception);
-          tests+=ValidateImageFormatsInMemory(image_info,reference_filename,
-            output_filename,&fail,exception);
-          tests+=ValidateImportExportPixels(image_info,reference_filename,
-            output_filename,&fail,exception);
+          (void) fprintf(stdout,"ImageMagick Validation Suite (%s)\n\n",
+            MagickOptionToMnemonic(MagickValidateOptions,(long) type));
+          if ((type & CompareValidate) != 0)
+            tests+=ValidateCompareCommand(image_info,reference_filename,
+              output_filename,&fail,exception);
+          if ((type & CompositeValidate) != 0)
+            tests+=ValidateCompositeCommand(image_info,reference_filename,
+              output_filename,&fail,exception);
+          if ((type & ConvertValidate) != 0)
+            tests+=ValidateConvertCommand(image_info,reference_filename,
+              output_filename,&fail,exception);
+          if ((type & FormatsInMemoryValidate) != 0)
+            tests+=ValidateImageFormatsInMemory(image_info,reference_filename,
+              output_filename,&fail,exception);
+          if ((type & FormatsOnDiskValidate) != 0)
+            tests+=ValidateImageFormatsOnDisk(image_info,reference_filename,
+              output_filename,&fail,exception);
+          if ((type & IdentifyValidate) != 0)
+            tests+=ValidateIdentifyCommand(image_info,reference_filename,
+              output_filename,&fail,exception);
+          if ((type & ImportExportValidate) != 0)
+            tests+=ValidateImportExportPixels(image_info,reference_filename,
+              output_filename,&fail,exception);
+          if ((type & MontageValidate) != 0)
+            tests+=ValidateMontageCommand(image_info,reference_filename,
+              output_filename,&fail,exception);
+          if ((type & StreamValidate) != 0)
+            tests+=ValidateStreamCommand(image_info,reference_filename,
+              output_filename,&fail,exception);
           (void) fprintf(stdout,"validation suite: %lu tests; %lu passed; "
             "%lu failed.\n",tests,tests-fail,fail);
         }
       (void) RelinquishUniqueFileResource(output_filename);
       (void) RelinquishUniqueFileResource(reference_filename);
     }
-  image_info=DestroyImageInfo(image_info);
   if (exception->severity != UndefinedException)
     CatchException(exception);
   if (iterations > 1)
@@ -1341,8 +1430,7 @@ int main(int argc,char **argv)
         iterations,1.0*iterations/elapsed_time,user_time,(long)
         (elapsed_time/60.0+0.5),(long) ceil(fmod(elapsed_time,60.0)));
     }
-  timer=DestroyTimerInfo(timer);
-  exception=DestroyExceptionInfo(exception);
+  DestroyValidate();
   MagickCoreTerminus();
   return(fail == 0 ? 0 : 1);
 }
