@@ -138,10 +138,8 @@ static const char
     "-label Magick",
     "-lat 10x10-5%",
     "-level 10%,1.2,90%",
-    "-magnify",
     "-map netscape:",
     "-median 2",
-    "-minify",
     "-modulate 110/100/95",
     "-monochrome",
     "-motion-blur 0x3+30",
@@ -198,6 +196,7 @@ static const char
   *identify_options[] =
   {
     "",
+    "-verbose",
     (const char *) NULL
   };
 
