@@ -33,14 +33,8 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Import is an X Window System window dumping utility.  Import allows X
-%  users to store window images in a specially formatted dump file.  This
-%  file can then be read by the Display utility for redisplay, printing,
-%  editing, formatting, archiving, image processing, etc.  The target
-%  window can be specified by id or name or be selected by clicking the
-%  mouse in the desired window.  The keyboard bell is rung once at the
-%  beginning of the dump and twice when the dump is completed.
-%
+%  Use the import program to capture some or all of an X server screen and
+%  save the image to a file.
 %
 */
 
@@ -342,8 +336,6 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
       Check command line for server name.
     */
     option=argv[i];
-    if (IsMagickOption(option) == MagickFalse)
-      continue;
     if (LocaleCompare("display",option+1) == 0)
       {
         /*
@@ -474,7 +466,6 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
           images->scene=scene;
           AppendImageStack(images);
         }
-        FireImageStack(MagickTrue,MagickFalse,MagickTrue);
         continue;
       }
     pend=image != (Image *) NULL ? MagickTrue : MagickFalse;
@@ -801,7 +792,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("frame",option+1) == 0)
           {
-            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans0",MaxTextExtent);
             ximage_info.frame=(*option == '-') ? MagickTrue : MagickFalse;
             break;
           }
@@ -1275,15 +1266,17 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
   }
   if (k != 0)
     ThrowImportException(OptionError,"UnbalancedParenthesis",argv[i]);
-  if (i != argc)
+  if (i-- != argc)
     ThrowImportException(OptionError,"MissingAnImageFilename",argv[i]);
   if (image == (Image *) NULL)
     ThrowImportException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  FinalizeImageSettings(image_info,image);
+  FinalizeImageSettings(image_info,image,MagickTrue);
   status&=WriteImages(image_info,image,filename,exception);
   DestroyImport();
   return(status != 0 ? MagickTrue : MagickFalse);
 #else
+  (void) argc;
+  (void) argv;
   (void) ThrowMagickException(exception,GetMagickModule(),MissingDelegateError,
     "DelegateLibrarySupportNotBuiltIn","`%s' (X11)",image_info->filename);
   return(ImportUsage());
