@@ -25,7 +25,6 @@ extern "C" {
 #define AppendImageStack(images) \
 { \
   (void) SyncImagesSettings(image_info,images); \
-  (void) DeleteImageOption(image_info,"page"); \
   AppendImageToList(&image_stack[k].image,images); \
   image=image_stack[k].image; \
 }
@@ -37,15 +36,10 @@ extern "C" {
   image_stack[k].image=DestroyImageList(image_stack[k].image); \
   image_stack[k].image_info=DestroyImageInfo(image_stack[k].image_info); \
 }
-#define FinalizeImageSettings(image_info,image) \
+#define FinalizeImageSettings(image_info,image,advance) \
 { \
-  (void) DeleteImageOption(image_info,"compress"); \
-  (void) DeleteImageOption(image_info,"delay"); \
-  (void) DeleteImageOption(image_info,"dispose"); \
-  (void) DeleteImageOption(image_info,"loop"); \
-  (void) DeleteImageOption(image_info,"page"); \
-  (void) DeleteImageOption(image_info,"tile-offset"); \
-  FireImageStack(MagickTrue,MagickTrue,MagickTrue); \
+  ResetImageOptions(image_info); \
+  FireImageStack(MagickTrue,advance,MagickTrue); \
   (void) SyncImagesSettings(image_info,image); \
 }
 #define FireImageStack(postfix,advance,fire) \
@@ -85,7 +79,6 @@ extern "C" {
 }
 #define PopImageStack() \
 { \
-  (void) DeleteImageOption(image_info,"page"); \
   if (respect_parenthesis == MagickFalse) \
     { \
       image_stack[k-1].image_info=DestroyImageInfo(image_stack[k-1].image_info); \
