@@ -33,10 +33,11 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Use Stream is a lightweight utility designed to extract pixels from large
-%  image files to a raw format using a minimum of system resources.  The
-%  entire image or any regular portion of the image can be extracted.
-%
+%  Stream is a lightweight tool to stream one or more pixel components of the
+%  image or portion of the image to your choice of storage formats. It writes
+%  the pixel components as they are read from the input image a row at a time
+%  making stream desirable when working with large images or when you require
+%  raw pixel components.
 %
 */
 
@@ -298,7 +299,6 @@ WandExport MagickBooleanType StreamImageCommand(ImageInfo *image_info,
         if (images == (Image *) NULL)
           continue;
         AppendImageStack(images);
-        FireImageStack(MagickTrue,MagickFalse,MagickTrue);
         continue;
       }
     pend=image != (Image *) NULL ? MagickTrue : MagickFalse;
@@ -739,7 +739,7 @@ WandExport MagickBooleanType StreamImageCommand(ImageInfo *image_info,
     ThrowStreamException(OptionError,"MissingAnImageFilename",argv[i]);
   if (image == (Image *) NULL)
     ThrowStreamException(OptionError,"MissingAnImageFilename",argv[i]);
-  FinalizeImageSettings(image_info,image);
+  FinalizeImageSettings(image_info,image,MagickTrue);
   DestroyStream();
   return(status != 0 ? MagickTrue : MagickFalse);
 }
